package org.loststone.toodledo {
import java.util.List;

import org.loststone.toodledo.data.Context;
import org.loststone.toodledo.data.Folder;
import org.loststone.toodledo.data.Goal;
import org.loststone.toodledo.data.Todo;
import org.loststone.toodledo.exception.IncorrectUserPasswordException;
import org.loststone.toodledo.exception.MissingPasswordException;
import org.loststone.toodledo.exception.ToodledoApiException;
import org.loststone.toodledo.request.AddContextRequest;
import org.loststone.toodledo.request.AddFolderRequest;
import org.loststone.toodledo.request.AddGoalRequest;
import org.loststone.toodledo.request.AddTodoRequest;
import org.loststone.toodledo.request.AuthorizeRequest;
import org.loststone.toodledo.request.DeleteTodoRequest;
import org.loststone.toodledo.request.GetContextsRequest;
import org.loststone.toodledo.request.GetFoldersRequest;
import org.loststone.toodledo.request.GetGoalsRequest;
import org.loststone.toodledo.request.GetTodosRequest;
import org.loststone.toodledo.request.GetUserIdRequest;
import org.loststone.toodledo.request.ModifyTodoRequest;
import org.loststone.toodledo.request.Request;
import org.loststone.toodledo.response.AddContextResponse;
import org.loststone.toodledo.response.AddFolderResponse;
import org.loststone.toodledo.response.AddGoalResponse;
import org.loststone.toodledo.response.AddTodoResponse;
import org.loststone.toodledo.response.AuthorizeResponse;
import org.loststone.toodledo.response.DeleteTodoResponse;
import org.loststone.toodledo.response.GetContextsResponse;
import org.loststone.toodledo.response.GetFoldersResponse;
import org.loststone.toodledo.response.GetGoalsResponse;
import org.loststone.toodledo.response.GetTodosResponse;
import org.loststone.toodledo.response.GetUserIdResponse;
import org.loststone.toodledo.response.ModifyTodoResponse;
import org.loststone.toodledo.util.AuthToken;
import org.loststone.toodledo.xml.ContextsParser;
import org.loststone.toodledo.xml.FolderParser;
import org.loststone.toodledo.xml.GetTodosParser;
import org.loststone.toodledo.xml.GetUserIdParser;
import org.loststone.toodledo.xml.GoalsParser;

public class ToodledoApiImpl implements ToodledoApi {

	public function addTodo(auth:AuthToken, todo:Todo):int throws ToodledoApiException {
		var request:AddTodoRequest= new AddTodoRequest(auth, todo);
		var resp:AddTodoResponse= AddTodoResponse(request.getResponse());
		if (resp.succeeded())
			return Integer.parseInt(resp.getResponseContent());
		else
			return -1;
	}

	public function getTodo(auth:AuthToken, id:int):Todo throws ToodledoApiException {
		var filter:Todo= new Todo();
		filter.setId(id);
		List<Todo> res = getTodosList(auth,filter);
		if (res != null && res.size() > 0) {
			return res.get(0);
		} else {
			return null;
		}
	}

	public List<Todo> getTodosList(var auth:AuthToken) throws ToodledoApiException {
		return getTodosList(auth,null);
	}
	
	public List<Todo> getTodosList(var auth:AuthToken, var filter:Todo) throws ToodledoApiException {
		var getTodosRequest:Request= new GetTodosRequest(auth, filter);
		var response:GetTodosResponse= GetTodosResponse(getTodosRequest.getResponse());
		if (response.succeeded())
			return new GetTodosParser(response.getXmlResponseContent()).getTodos();
		else
			return null;
	}

	public function initialize(username:String, password:String):AuthToken throws ToodledoApiException {
		var initReq:Request= new AuthorizeRequest(username);
		// response gives back the token, now create the AuthToken
		var resp:AuthorizeResponse= AuthorizeResponse(initReq.getResponse());
		var token:AuthToken= new AuthToken(password, username, resp.getResponseContent());
		return token;
	}

	public function modifyTodo(auth:AuthToken, newOne:Todo):Boolean throws ToodledoApiException{
		var modifyRequest:ModifyTodoRequest= new ModifyTodoRequest(auth,newOne);
		var resp:ModifyTodoResponse= ModifyTodoResponse(modifyRequest.getResponse());
		if (resp.succeeded()) {
			var _t:Integer= Integer.parseInt(resp.getResponseContent());
			if (_t == 1) return true;
			else return false;
		} else
			return false;
	}

	public function deleteTodo(auth:AuthToken, id:int):Boolean throws ToodledoApiException{
		var request:DeleteTodoRequest= new DeleteTodoRequest(auth, id);
		var resp:DeleteTodoResponse= DeleteTodoResponse(request.getResponse());
		if (resp.succeeded()) {
			var _t:Integer= Integer.parseInt(resp.getResponseContent());
			if (_t == 1) return true;
			else return false;
		} else
			return false;
	}

	
	public List<Context> getContexts(var auth:AuthToken)  throws ToodledoApiException{
		var request:GetContextsRequest= new GetContextsRequest(auth);
		var resp:GetContextsResponse= GetContextsResponse(request.getResponse());
		if (resp.succeeded())
			return new ContextsParser(resp.getXmlResponseContent()).getContexts();
		else
			return null;
	}

	public List<Folder> getFolders(var auth:AuthToken)  throws ToodledoApiException{
		var request:GetFoldersRequest= new GetFoldersRequest(auth);
		var resp:GetFoldersResponse= GetFoldersResponse(request.getResponse());
		if (resp.succeeded())
			return new FolderParser(resp.getXmlResponseContent()).getFolders();
		else
			return null;
	}

	public List<Goal> getGoals(var auth:AuthToken)  throws ToodledoApiException{
		var request:GetGoalsRequest= new GetGoalsRequest(auth);
		var resp:GetGoalsResponse= GetGoalsResponse(request.getResponse());
		if (resp.succeeded())
			return new GoalsParser(resp.getXmlResponseContent()).getGoals();
		else
			return null;
	}

	public function addFolder(auth:AuthToken, fold:Folder):int throws ToodledoApiException {
		var request:AddFolderRequest= new AddFolderRequest(auth,fold);
		var response:AddFolderResponse= AddFolderResponse(request.getResponse());
		if (response.succeeded())
			return Integer.parseInt(response.getResponseContent());
		else
			return -1;
	}

	public function addContext(auth:AuthToken, context:Context):int throws ToodledoApiException {
		var request:AddContextRequest= new AddContextRequest(auth, context);
		var response:AddContextResponse= AddContextResponse(request.getResponse());
		if (response.succeeded())
			return Integer.parseInt(response.getResponseContent());
		else
			return -1;
	}

	public function addGoal(auth:AuthToken, goal:Goal):int throws ToodledoApiException {
		var request:AddGoalRequest= new AddGoalRequest(auth, goal);
		var response:AddGoalResponse= AddGoalResponse(request.getResponse());
		if (response.succeeded())
			return Integer.parseInt(response.getResponseContent());
		else
			return -1;
	}

	public function getUserId(mail:String, password:String):String throws ToodledoApiException, IncorrectUserPasswordException, MissingPasswordException {
		var request:GetUserIdRequest= new GetUserIdRequest(mail,password);
		var response:GetUserIdResponse= GetUserIdResponse(request.getResponse());
		if(response.succeeded()) 
			return new GetUserIdParser(response.getXmlResponseContent()).getUserId();
		else
			return null;
	}

}
}