﻿package org.toodledo {
/**
 * This class represents a folder in toodledo.
 * @author lant
 *
 */
public class Folder {
	var _id:int; 
	// is private ? 
	var _bPrivate:Boolean;
	// is it archived ? 
	var _archived:Boolean;
	// that's the user prefered order for listing the folders.
	var _order:int;
	var _sName:String;
	
	var _hasName:Boolean= false;
	var _hasPrivate:Boolean= false;
	
	/**
	 * Checks if this folder has the name field set.
	 * @return true if this folder has the name field correctly set.
	 */
	public function hasName():Boolean {
		return this._hasName;
	}
	
	/**
	 * Check if the private field is set.
	 * @return Does not return if this folder is private or not, it checks if the private field
	 * is set.
	 */
	public function hasPrivate():Boolean {
		return this._hasPrivate;
	}
	
	/**
	 * Returns the internal id of this folder.
	 * @return the id
	 */
	public function getId():int {
		return _id;
	}
	
	/**
	 * Sets the internal id of this folder.
	 * This should only be used when creating the folder from a REST response, as the id is set
	 * internally in toodledo.
	 * @param id to set
	 */
	public function setId(id:int):void {
		this._id = id;
	}
	
	/**
	 * Checks if this folder is private.
	 * @return true if the folder is set to private, false otherwise.
	 */
	public function isBPrivate():Boolean {
		return _bPrivate;
	}
	
	/**
	 * @param true if the folder is to be set to private, false otherwise.
	 */
	public function setBPrivate(private1:Boolean):void {
		_bPrivate = private1;
		_hasPrivate = true;
	}
	
	/**
	 * @return true if the folder is archived, false otherwise.
	 */
	public function isArchived():Boolean {
		return _archived;
	}
	
	/**
	 * @param true if the folder is archived, false otherwise.
	 */
	public function setArchived(archived:Boolean):void {
		this._archived = archived;
	}
	
	/**
	 * Returns the order of this folder when it's listed.
	 * @return the order
	 */
	public function getOrder():int {
		return _order;
	}
	
	/**
	 * Sets the order for this folder. 
	 * @param order to set.
	 */
	public function setOrder(order:int):void {
		this._order = order;
	}
	
	/**
	 * Gets the name of the folder.
	 * @return the name of the folder.
	 */
	public function getSName():String {
		return _sName;
	}
	
	/**
	 * Sets the name of the folder.
	 * @param name the folder. If the name is longer than 32 chars it will be cropped.
	 */
	public function setSName(name:String):void {
		if (name.length > 32)
			_sName = name.substring(0,31);
		else
			_sName = name;
		
		this._hasName = true;
	}
}
}